/*
 * RFID 读写卡测试程序 DFrobot
 *   MFRC522 模块
 *   MRC50-1 白卡
 * 模块资料：http://wiki.dfrobot.com.cn/index.php?title=RFID_Reader_Module%28.NET_Gadgeteer_Compatible%29_%28SKU:TOY0019%29
 */

#include "RC522.h"

/* 特别重要!! 
 * 如果使用的IDE版本小于1.0，请务必将下行注释掉
*/
#define NEWIDE 


/////////////////////////////////////////////////////////////////////
//复位引脚定义
/////////////////////////////////////////////////////////////////////
#define RC522_RST  34//Dreamer MEGA  X14 port PIN6

#define SET_RC522RST  digitalWrite(RC522_RST,HIGH)
#define CLR_RC522RST  digitalWrite(RC522_RST,LOW)


unsigned char RevBuffer[30];
unsigned char CardID[4];//存储卡片序列号
unsigned char Read_Data[16]={0x00};
unsigned char PassWd[6]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};//密码
unsigned char WriteData[16]={0};
unsigned char NewKey[16]={0x00};//存储新密码

unsigned char BlockN=0;
unsigned char Command;
unsigned char N;
unsigned char Wdata;

/////////////////////////////////////////////////////////////////////
//功    能：初始化RC522
//
/////////////////////////////////////////////////////////////////////
void InitRc522(void)
{
  PcdReset();
  PcdAntennaOff();  
  PcdAntennaOn();
  M500PcdConfigISOType( 'A' );
}
/////////////////////////////////////////////////////////////////////
//功    能：操作MFRC522
/////////////////////////////////////////////////////////////////////
void CtrlRc522(void)
{
     char stat;
     unsigned char ii;
     
     if(Command=='Q')//寻卡
     {
           PcdReset();
           delay(100);
           
           stat=PcdRequest(PICC_REQIDL,&RevBuffer[0]);//寻天线区内未进入休眠状态的卡，返回卡片类型 2字节
           if(stat!=MI_OK)
           {
              return;
           }
           //发送卡片类型代码
           Serial.print("Card type code; ");
           Serial.print(RevBuffer[0],DEC);
           Serial.print(',');
           Serial.println(RevBuffer[1],DEC);
           Serial.println();
     }
     else if(Command=='A')//防冲撞及选卡
     {
           stat=PcdAnticoll(&RevBuffer[2]);//防冲撞，返回卡的序列号 4字节
           if(stat!=MI_OK)
           {
              return;
           }
           memcpy(CardID,&RevBuffer[2],4); 
           stat=PcdSelect(CardID);//选卡
           if(stat!=MI_OK)
           {
               return;
           }
           //发送卡片序列号
           Serial.print("Card series number; ");
           Serial.print(CardID[0],DEC);
           Serial.print(',');
           Serial.print(CardID[1],DEC);
           Serial.print(',');
           Serial.print(CardID[2],DEC);
           Serial.print(',');
           Serial.println(CardID[3],DEC);
           Serial.println();
     }
     else if(Command=='R' && (BlockN>=0 && BlockN<=15))//读卡
     {
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdRead(BlockN,Read_Data);//读取块N数据
           if(stat!=MI_OK)
           {
               return;
           }
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。
           //发送卡片块N的内容
           Serial.print("Card block");
           Serial.print(BlockN,DEC);
           Serial.print(" content; ");
           for(ii=0;ii<15;ii++)
           {
               Serial.print(Read_Data[ii],DEC);
               Serial.print(',');
           }
           Serial.println(Read_Data[15],DEC);
           Serial.println();
     }
     else if(Command=='W' && ((BlockN>0 && BlockN<15) && BlockN!=3 && BlockN!=7 && BlockN!=11))//写卡
     {
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdWrite(BlockN,&WriteData[0]);//向块N写数据
           if(stat!=MI_OK)
           {
               return;
           }	
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。
           //发送卡片块N写OK
           Serial.print("Card block");
           Serial.print(BlockN,DEC);
           Serial.println(" Write ok!");
           Serial.println();
     }  
     //密码A永远不可读，读出数据显示为“Ox000000”,因此一旦设定就必须记住；一定条件下可改写，详见手册
     //密码B在一定条件下可读写，详见手册
    /*else if(Command=='K')//修改密码
     {   
           stat=PcdAuthState(PICC_AUTHENT1A,BlockN,PassWd,CardID);//验证密码
           if(stat!=MI_OK)
           {
               return;
           }
           stat=PcdWrite(BlockN,&NewKey[0]);//写新密码
           if(stat!=MI_OK)
           {
               return;
           }
           //PcdHalt();//卡片休眠，不响应读写模块的任何命令；可使用寻卡命令唤醒。		
     }*/

}
void setup()
{
     //设定端口模式
     pinMode(RC522_RST,OUTPUT);
     SET_RC522RST;
     
     //串口初始化
     Serial1.begin(9600);//串口2：主控板与模块通信
     Serial.begin(9600);//串口：主控板与USB通信
     delay(1000);
}
void loop()
{
     unsigned char i;
     
     InitRc522();//初始化
     delay(100);
     Serial.println("The initialization completes.  ");
     Serial.println();
     
     while(1)
     {   
         if(Serial.available() > 0) 
         {
             Command=Serial.read();//接收命令字符
             delay(10);
             if(Command=='R'|| Command=='W')
             {
                   N=Serial.read();
                   
                   if(Command=='W')
                   {
                      delay(10);
                      Wdata=Serial.read();
                      Wdata -='0';
                      for(i=0;i<16;i++)
                      {
                         WriteData[i]=Wdata;
                      }
                   }
                   
                   switch(N)
                   {
                     case '0': BlockN=0;
                               break;
                     case '1': BlockN=1;
                               break;
                     case '2': BlockN=2;
                               break;   
                     case '3': BlockN=3;
                               break;
                     case '4': BlockN=4;
                               break;
                     case '5': BlockN=5;
                               break;   
                     case '6': BlockN=6;
                               break;
                     case '7': BlockN=7;
                               break;
                     case '8': BlockN=8;
                               break; 
                     case '9': BlockN=9;
                               break;
                     case 'A': BlockN=10;
                               break;
                     case 'B': BlockN=11;
                               break;
                     case 'C': BlockN=12;
                               break;
                     case 'D': BlockN=13;
                               break;
                     case 'E': BlockN=14;
                               break;  
                     case 'F': BlockN=15;
                               break;                             
                     default:  Serial.println("Error!");
                               Serial.println();
                               BlockN=16;
                   }
             }
             
             CtrlRc522();//操作MFRC522
         }
     }
     
}
